/**********************************************************************
 Freeciv - Copyright (C) 2003  - M.C. Kaufman
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <signal.h>

#include <sys/wait.h>
#include <sys/types.h>
#include <unistd.h>

#include "user_db.h"

#define USER_DB_FILE "freeciv_db"

#ifdef USER_DB_PROGPATH
  #define USER_DB_PROGRAM USER_DB_PROGPATH "/" USER_DB_FILE
#else
  #define USER_DB_PROGRAM USER_DB_FILE
#endif

/* the function to send our lookup back to the server */
extern void handle_db_lookup(const char *username, enum userdb_status status);

static void sig_handler(int sig);

static char user[32];

/**************************************************************************
 catches the signal that the child user database program emits when it exits.
 hands off to the higher-level handle_db_authentication().
**************************************************************************/
void sig_handler(int sig)
{
  enum userdb_status status;

  wait((int *)&status);

  assert(WEXITSTATUS(status) != USER_DB_BAD_INPUT);

  printf("exited = %d, status = %d\n", 
         WIFEXITED(status), WEXITSTATUS(status));fflush(stdout);/* DEBUGGING */

  handle_db_lookup(user, WEXITSTATUS(status));
}

/**************************************************************************
 Does a search through the database for the given username to see if it
 exists.
**************************************************************************/
void user_db_search(const char *username)
{
  pid_t pid;

  printf("search: u:%s\n", username);fflush(stdout);

  strncpy(user, username, 32);
  signal(SIGCHLD, sig_handler);

  pid = fork();

  if (pid == 0) {
    close(0);
    close(1);

    execlp(USER_DB_PROGRAM, USER_DB_FILE,
           username, "", "1", NULL);

    /* this will happen if we can't run the program */
    exit(USER_DB_ERROR);
  }
}

/**************************************************************************
 Does a search through the database for the givens name and matching password.
**************************************************************************/
void user_db_check(const char *username, const char *password)
{
  pid_t pid;

  printf("check: u:%s p:%s\n", username, password);fflush(stdout);

  strncpy(user, username, 32);
  signal(SIGCHLD, sig_handler);

  pid = fork();

  if (pid == 0) {
    close(0);
    close(1);

    execlp(USER_DB_PROGRAM, USER_DB_FILE,
           username, password, "1", NULL);

    /* this will happen if we can't run the program */
    exit(USER_DB_ERROR);
  }
}

/**************************************************************************
 Create a new entry in the user database from pconn->username and 
 pconn->password
**************************************************************************/
void user_db_create(const char *username, const char *password)
{
  pid_t pid;

  /* DEBUGGING FIXME: remove before commit! */ 
  printf("create: u:%s p:%s\n", username, password);fflush(stdout);

  strncpy(user, username, 32);
  signal(SIGCHLD, sig_handler);

  pid = fork();

  if (pid == 0) {
    close(0);
    close(1);

    execlp(USER_DB_PROGRAM, USER_DB_FILE, 
           username, password, "0", NULL);

    /* this will happen if we can't run the program */
    exit(USER_DB_ERROR);
  }  
}
